package gephi;

import graph.Edge;
import graph.Graph;
import graph.GraphFile;
import graph.Vertex;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.zip.DataFormatException;

public class Gexf extends GraphFile
{

	///////////////
	// CONSTANTS //
	///////////////
	
	private static final String FILE_EXT = ".gexf";
	
	
	

	////////////
	// FIELDS //
	////////////
	
	
	
	
	

	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException
	 * @throws DataFormatException
	 */
	public Gexf(File inputFile) throws IOException, DataFormatException
	{
		super(inputFile);
	}
	
	/**
	 * 
	 * @param graph
	 */
	public Gexf(Graph graph)
	{
		super(graph);
	}
	
	/**
	 * 
	 * @param gf
	 */
	public Gexf(GraphFile gf)
	{
		super(gf);
	}

	
	
	

	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////

	/**
	 * 
	 * @return
	 */
	public String getFileExt()
	{
		return FILE_EXT;
	}
	
	
	
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * 
	 */
	@Override
	public void readFromFile(File inputFile) throws IOException
	{
		// TODO Auto-generated method stub
		
	}

	/**
	 * 
	 */
	@Override
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = null;
		try
		{
			writer = new BufferedWriter(new FileWriter(outputFile));
			writeFile(writer);
		}
		finally
		{
			if(writer != null)
				writer.close();
		}
	}




	


	

	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * 
	 * @param writer
	 * @throws IOException 
	 */
	private void writeFile(BufferedWriter writer) throws IOException
	{
		writer.write("<gexf>");
		writer.newLine();
		writeGraph(writer, "\t");
		writer.write("</gexf>");
	}



	/**
	 * 
	 * @param writer
	 * @throws IOException 
	 */
	private void writeGraph(BufferedWriter writer, String indent) throws IOException
	{
		writer.write(indent + "<graph defaultedgetype=\"undirected\">");
		writer.newLine();
		writeGraphVertAttrs(writer, indent + "\t");
		writeVerts(writer, indent + "\t");
		writeEdges(writer, indent + "\t");
		writer.write(indent + "</graph>");
		writer.newLine();
	}

	/**
	 * 
	 */
	private void writeGraphVertAttrs(BufferedWriter writer, String indent) throws IOException
	{
		writer.write(indent + "<attributes class=\"node\" mode=\"static\">");
		writer.newLine();
		for(Vertex vert : graph.getVertices())
		{
			writeVert(vert, writer, indent + "\t");
		}
		writer.write(indent + "</attributes>");
		writer.newLine();
	}

	/**
	 * 
	 * @param writer
	 * @param indent
	 * @throws IOException
	 */
	private void writeVerts(BufferedWriter writer, String indent) throws IOException
	{
		writer.write(indent + "<nodes>");
		writer.newLine();
		for(Vertex vert : graph.getVertices())
		{
			writeVert(vert, writer, indent + "\t");
		}
		writer.write(indent + "</nodes>");
		writer.newLine();
	}



	/**
	 * 
	 * @param writer
	 * @param indent
	 * @throws IOException
	 */
	private void writeEdges(BufferedWriter writer, String indent) throws IOException
	{

		writer.write(indent + "<edges>");
		writer.newLine();
		for(Edge edge : graph.getEdges())
		{
			writeEdge(edge, writer, indent + "\t");
		}
		writer.write(indent + "</edges>");
		writer.newLine();
	}

	/**
	 * 
	 * @param vert
	 * @param writer
	 * @param indent
	 * @throws IOException
	 */
	private void writeVert(Vertex vert, BufferedWriter writer, String indent) throws IOException
	{
		StringBuffer sb = new StringBuffer();
		sb.append(indent + "<node ");
		sb.append("id=\"" + vert.getId() + "\" ");
		sb.append("label=\"" + vert.getAttrVal("label") + "\"");
		sb.append("/>");
		writer.write(sb.toString());
		writer.newLine();	
	}

	/**
	 * 
	 * @param edge
	 * @param writer
	 * @param indent
	 * @throws IOException
	 */
	private void writeEdge(Edge edge, BufferedWriter writer, String indent) throws IOException
	{
		StringBuffer sb = new StringBuffer();
		sb.append(indent + "<edge ");
		sb.append("id=\"" + edge.getId() + "\" ");
		sb.append("source=\"" + edge.getVert1Id() + "\" ");
		sb.append("target=\"" + edge.getVert2Id() + "\"");
		sb.append("/>");
		writer.write(sb.toString());
		writer.newLine();
	}
	
	
	
	//////////////////////
	// STANDARD METHODS //
	//////////////////////

}
