package gephi;

import graph.*;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;





//TODO: Use GraphSequence
public class Rec 
{
	
	///////////////
	// CONSTANTS //
	///////////////
	
	private static final int VERTEX_SIZE = 4;
	private static final int MIN_EDGE_THICKNESS = 2;
	

	
	
	
	////////////
	// FIELDS //
	////////////
	
	private List<Graph> graphs;
	private List<String> events;
	private Graph currGraph;
	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	public Rec() throws IOException
	{
		this.currGraph = new Graph();
		this.events = new ArrayList<String>();
	}
	
	
	
	//TODO: Probably obsolete
	public Rec(List<Graph> graphs) throws IOException
	{
		this.graphs = graphs;
		this.currGraph = new Graph();
		this.events = new ArrayList<String>();
		this.init();
	}




	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	
	public void nextGraph(Graph nextGraph)
	{
		this.events = new ArrayList<String>();
		recordDeleteEvents(nextGraph);
		recordAddEvents(nextGraph);
		currGraph = nextGraph;
	}
	
	
	/**
	 * 
	 * @param nextGraph
	 */
	public void recordDeleteEvents(Graph nextGraph)
	{
		Edges currEdges = currGraph.getEdges();
		Edges nextEdges = nextGraph.getEdges();
		Iterator<Edge> edgeItr = currEdges.iterator();
		while(edgeItr.hasNext())
		{
			Edge currEdge = edgeItr.next();
			if(!nextEdges.contains(currEdge))
			{
				deleteEdge(currEdge);
				edgeItr.remove();
			}
		}
		
		Vertices currVerts = currGraph.getVertices();
		Vertices nextVerts = nextGraph.getVertices();
		Iterator<Vertex> vertItr = currVerts.iterator();
		while(vertItr.hasNext())
		{
			Vertex currVert = vertItr.next();
			if(!nextVerts.contains(currVert))
			{
				deleteNode(currVert);
				vertItr.remove();
			}
		}
	}
	
	/**
	 * 
	 * @param wadj
	 */
	public void recordAddEvents(Graph graph) 
	{	
		Vertices currVertices = currGraph.getVertices();
		for(Edge edge : graph.getEdges())
		{
			if(!currVertices.contains(edge.getVert1()))
				addNode(edge.getVert1());
			if(!currVertices.contains(edge.getVert2()))
				addNode(edge.getVert2());
			addEdge(edge);
		}
	}
	
	
	
	/**
	 * 
	 * @param edge
	 */
	private void deleteEdge(Edge edge)
	{
		StringBuffer sb = new StringBuffer();
		sb.append("{\"de\":");
		sb.append("{\"" + edge.getId() + "\"");
		sb.append("}}");
		events.add(sb.toString());
	}
	
	/**
	 * 
	 * @param vertex
	 */
	private void deleteNode(Vertex vertex)
	{
		StringBuffer sb = new StringBuffer();
		sb.append("{\"dn\":");
		sb.append("{\"" + vertex.getId() + "\"");
		sb.append("}}");
		events.add(sb.toString());
	}




	
	
	/**
	 * 
	 * @param vertex
	 */
	private void addNode(Vertex vertex) 
	{
		Attributes attrs = vertex.getAttrs();
		
		currGraph.addVertex(vertex);
		StringBuffer sb = new StringBuffer();
		sb.append("{\"an\":");
		sb.append("{\"" + vertex.getId() + "\":");
		sb.append("{\"size\":\"" + VERTEX_SIZE + "\"");
		
		for(String attrId : attrs.getIds())
		{
			sb.append("\"" + attrId + "\":\"" + attrs.getValue(attrId) + "\"");
		}
		
		
//		sb.append("\"x\":\"" + vertex.getAttrs().get("x") + "\"");
//		sb.append(",\"y\":\"" + vertex.getAttrs().get("y") + "\"");
//		sb.append(",\"label\":\"" + vertex.getId() + "\"");
//		sb.append(",\"r\":\"" + vertex.getAttrs().getValue("r") + "\"");
//		sb.append(",\"g\":\"" + vertex.getAttrs().getValue("g") + "\"");
//		sb.append(",\"b\":\"" + vertex.getAttrs().getValue("b") + "\"");
		
		
		sb.append("}}}");
		events.add(sb.toString());
	}
	
	/**
	 * 
	 * @param edge
	 */
	private void addEdge(Edge edge) 
	{
		
		Attributes attrs = edge.getAttrs();
		
		double r = Double.parseDouble(attrs.getValue("r"))/255;
		double g = Double.parseDouble(attrs.getValue("g"))/255;
		double b = Double.parseDouble(attrs.getValue("b"))/255;
		
		currGraph.addEdge(edge);
		StringBuffer sb = new StringBuffer();
		sb.append("{\"ae\":");
		sb.append("{\"" + edge.getId() + "\":");
		sb.append("{\"source\":\"" + edge.getVert1Id() + "\"");
		sb.append(",\"target\":\"" + edge.getVert2Id() + "\"");
		
		
		
		
		sb.append(",\"r\":\"" + r + "\"");
		sb.append(",\"g\":\"" + g + "\"");
		sb.append(",\"b\":\"" + b + "\"");
		
		//sb.append(",\"directed\":false");
		
		
		sb.append("}}}");
		events.add(sb.toString());
	}
	
	
	/**
	 * 
	 * @param outputFile
	 * @throws IOException
	 */
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));
		for(String event : events)
		{
			writer.write(event);
			writer.newLine();
			writer.flush();
		}
	}





	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * 
	 */
	private void init()
	{
		for(Graph graph : graphs)
		{
			recordDeleteEvents(graph);
			recordAddEvents(graph);
		}
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
}
