package graph;

import java.util.Scanner;





public class Edge extends GraphElem
{
	
	///////////////
	// CONSTANTS //
	///////////////
	
	private static final String EDGE_ID_DELIMITER = "-";
	
	
	
	
	////////////
	// FIELDS //
	////////////
	
	private double weight;
	private final Vertex vert1;
	private final Vertex vert2;
	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 * @param vert1
	 * @param vert2
	 * @param weight
	 */
	public Edge(Vertex vert1, Vertex vert2, double weight) 
	{
		super(createId(vert1.getId(), vert2.getId()), GraphElemType.EDGE);
		this.vert1 = vert1;
		this.vert2 = vert2;	
		this.weight = weight;
	}
	
	/**
	 * @param vert1
	 * @param vert2
	 */
	public Edge(Vertex vert1, Vertex vert2) 
	{
		this(vert1, vert2, 0);
	}
	
	/**
	 * 
	 * @param vert1Id
	 * @param vert2Id
	 * @param weight
	 */
	public Edge(String vert1Id, String vert2Id, double weight)
	{
		this(new Vertex(vert1Id), new Vertex(vert2Id), weight);
	}
	
	/**
	 * 
	 * @param vert1Id
	 * @param vert2Id
	 */
	public Edge(String vert1Id, String vert2Id)
	{
		this(vert1Id, vert2Id, 0);
	}
	
	/**
	 * 
	 * @param edgeId
	 * @param weight
	 */
	public Edge(String edgeId, double weight)
	{
		this(getVert1Id(edgeId), getVert2Id(edgeId), weight);
	}
	
	/**
	 * 
	 * @param edgeId
	 * @param weight
	 */
	public Edge(String edgeId)
	{
		this(edgeId, 0);
	}
	
	
	
	
	
	
	
	

	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////
	
	/**
	 * @return the vert1
	 */
	public Vertex getVert1()
	{
		return vert1;
	}
	
	/**
	 * @return the vert2
	 */
	public Vertex getVert2()
	{
		return vert2;
	}
	
	/**
	 * 
	 * @return
	 */
	public String getVert1Id()
	{
		return vert1.getId();
	}
	
	/**
	 * 
	 * @return
	 */
	public String getVert2Id()
	{
		return vert2.getId();
	}

	/**
	 * @return the weight
	 */
	public double getWeight()
	{
		return weight;
	}

	/**
	 * @param weight the weight to set
	 */
	public void setWeight(double weight)
	{
		this.weight = weight;
	}
	
	
	
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * Constructs an edgeId from the specified vertexIds.
	 * @param vert1Id
	 * @param vert2Id
	 */
	public static String createId(String vert1Id, String vert2Id)
	{
		return(vert1Id + EDGE_ID_DELIMITER + vert2Id);
	}
	
	/**
	 * 
	 * @param edgeId
	 * @return
	 */
	public static String getVert1Id(String edgeId)
	{
		int delimiterIndex = edgeId.indexOf(EDGE_ID_DELIMITER);
		if(delimiterIndex < 0)
			return null;
		return edgeId.substring(0, delimiterIndex);
	}
	
	/**
	 * 
	 * @param edgeId
	 * @return
	 */
	public static String getVert2Id(String edgeId)
	{
		int delimiterIndex = edgeId.indexOf(EDGE_ID_DELIMITER);
		if(delimiterIndex < 0)
			return null;
		return edgeId.substring(delimiterIndex+1, edgeId.length());
	}
	
	
	
	
	
	
	//////////////////////
	// STANDARD METHODS //
	//////////////////////
	
	@Override
	public String toString()
	{
		return (vert1.getId() + "-" + vert2.getId() + " (" +  getWeight() + ") " + attrs);
	}

	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + ((vert1 == null) ? 0 : vert1.hashCode());
		result = prime * result + ((vert2 == null) ? 0 : vert2.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		Edge other = (Edge) obj;
		if (vert1 == null)
		{
			if (other.vert1 != null) return false;
		}
		else if (!vert1.equals(other.vert1)) return false;
		if (vert2 == null)
		{
			if (other.vert2 != null) return false;
		}
		else if (!vert2.equals(other.vert2)) return false;
		return true;
	}
	
	
	
	
	
	
	
	
	

	
	
	
	
}
