package graph;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.TreeMap;


/**
 * 
 */
public class Edges implements Iterable<Edge>
{

	////////////
	// FIELDS //
	////////////
	
	//TODO: Should be renamed to edgeMap
	private final Map<String,Map<String,Edge>> edges;
	

	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 */
	public Edges()
	{
		this.edges = new TreeMap<String,Map<String,Edge>>();
	}
	
	
	
	
	

	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////

	
	//TODO: Both are problematic
	
	/**
	 * 
	 * @param vert1Id
	 * @param vert2Id
	 * @return
	 */
	Edge getEdge(String vert1Id, String vert2Id)
	{
		Map<String,Edge> edgeMap = edges.get(vert1Id);
		if(edgeMap != null)
			return edgeMap.get(vert2Id);
		return null;
	}
	
	/**
	 * 
	 * @param edgeId
	 * @return
	 */
	Edge getEdge(String edgeId)
	{
		return getEdge(Edge.getVert1Id(edgeId), Edge.getVert2Id(edgeId));
	}
	
	
	
	
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * 
	 * @param edge
	 */
	boolean add(Edge edge)
	{
		if(edge == null || this.contains(edge))
			return false;
		String vert1Id = edge.getVert1Id();
		String vert2Id = edge.getVert2Id();
		if(!edges.containsKey(vert1Id))
			edges.put(vert1Id, new TreeMap<String,Edge>());
		Map<String,Edge> edgeMap = edges.get(vert1Id);
		edgeMap.put(vert2Id, edge);
		return true;
	}
	
	/**
	 * 
	 * @param edge
	 * @return
	 */
	Edge remove(Edge edge)
	{
		Map<String,Edge> edgeMap  = edges.get(edge.getVert1Id());
		if(edgeMap != null)
		{
			Edge removedEdge = edgeMap.remove(edge.getVert2Id());
			if(edgeMap.isEmpty())
			{
				edges.remove(edge.getVert1Id());
			}
			return removedEdge;
		}
		return null;
	}
	
	/**
	 * 
	 * @param edge
	 * @return
	 */
	//TODO: getEdge already does this
	public boolean contains(Edge edge)
	{
		Map<String,Edge> adjacentEdges = edges.get(edge.getVert1Id());
		if(adjacentEdges == null)
			return false;
		return adjacentEdges.containsKey(edge.getVert2Id());
	}
	
	/**
	 * 
	 * @param edgeId
	 * @return
	 */
	public boolean containsId(String edgeId)
	{
		return(getEdge(edgeId) != null);
	}
	
	/**
	 * 
	 * @param vert1Id
	 * @param vert2Id
	 * @return
	 */
	public boolean containsId(String vert1Id, String vert2Id)
	{
		return this.containsId(Edge.createId(vert1Id, vert2Id));
	}

	/**
	 * 
	 */
	public int size()
	{
		int size = 0;
		for(Map<String,Edge> edgeMap : edges.values())
		{
			size += edgeMap.size();
		}
		return size;
	}

	/**
	 * 
	 * @param atf
	 */
	void setAttrs(Atf atf)
	{
		for(String edgeId : atf.getElemIds())
		{
			Edge edge = this.getEdge(edgeId);
			if(edge != null)
				edge.setAttrs(atf.getAttrs(edgeId));
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public Collection<String> getVert1Ids()
	{
		return edges.keySet();
	}
	
	/**
	 * Gets the edges incident to the vertex with the specified
	 * vertId.
	 * @param vertId the id of the vertex to get incident edges.
	 * @return
	 */
	public Collection<Edge> getIncidentEdges(String vertId)
	{
		Map<String,Edge> edgeMap = edges.get(vertId);
		if(edgeMap != null)
			return edgeMap.values();
		return null;
	}
	
	/**
	 * 
	 */
	@Override
	public Iterator<Edge> iterator()
	{
		return new EdgeIterator(edges);
	}
	
	
	
	

	
	/////////////////////
	// PRIVATE METHODS //
	/////////////////////

	
	//////////////////////
	// STANDARD METHODS //
	//////////////////////
	
	////////////////////
	// NESTED CLASSES //
	////////////////////
	
	
	//TODO: Variable names are confusing.
	private class EdgeIterator implements Iterator<Edge>
	{
		
		private final Map<String,Map<String,Edge>> edges;
		private final List<Edge> edgeList;
		private Edge prev;
		private int pos;
		private int prevPos;
		
		
		
		/**
		 * 
		 * @param edges
		 */
		public EdgeIterator(Map<String,Map<String,Edge>> edges)
		{
			this.pos = 0;
			this.edges = edges;
			this.prevPos = -1;
			this.edgeList = new ArrayList<Edge>();
			for(Map<String,Edge> edgeMap : edges.values())
			{
				edgeList.addAll(edgeMap.values());
			}
		}
		
		/**
		 * 
		 */
		@Override
		public boolean hasNext()
		{
			return (pos < edgeList.size());
		}

		/**
		 * 
		 */
		@Override
		public Edge next() throws NoSuchElementException
		{
			Edge next = null;
			if(this.hasNext())
			{
				next = edgeList.get(pos);
				prevPos = pos;
				pos++;
			}
			else
				throw new NoSuchElementException();
			prev = next;
			return next;
		}

		/**
		 * 
		 */
		@Override
		public void remove() throws IllegalStateException
		{
			if(prev == null)
				throw new IllegalStateException();
			edges.remove(prev);
			edgeList.remove(prevPos);
			prev = null;
			pos--;
			prevPos = -1;
		}
	}

}
