/**
 * 
 */
package graph;


import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;

import graph.graph_filter.*;




/**
 * 
 *
 */
public class Graph extends GraphElem
{
	
	///////////////
	// CONSTANTS //
	///////////////
	
	public static final String DEFAULT_ID = "DefaultID";
	
	
	
	
	
	////////////
	// FIELDS //
	////////////
	
	private final Vertices vertices;
	private final Edges edges;

	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 */
	public Graph(String id)
	{
		super(id, GraphElemType.GRAPH);
		vertices = new Vertices();
		edges = new Edges();
	}
	
	/**
	 * 
	 */
	public Graph()
	{
		this(DEFAULT_ID);
	}
	
	
	
	
	
	
	
	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////
	
	/**
	 * 
	 * @return
	 */
	public Vertices getVertices() 
	{
		return vertices;
	}

	/**
	 * 
	 * @return
	 */
	public final Edges getEdges() 
	{
		return edges;
	}

	/**
	 * 
	 * @param atf
	 */
	public void setAttrs(Atf atf)
	{
		switch(atf.getType())
		{
			case GRAPH:  super.setAttrs(atf.getAttrs(id));
						 return;
			case VERTEX: vertices.setAttrs(atf);
						 return;
			case EDGE:	 edges.setAttrs(atf);
						 return;
						 
			//TODO: Should this throw an exception instead?
			default:	 System.err.println("No GraphElemType specified in Atf.");
						 return;
		}
	}
	
	
	
	
	
	
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * 
	 * @param vertex
	 */
	public void addVertex(Vertex vertex)
	{
		vertices.add(vertex);
	}
	
	/**
	 * 
	 * @param vertId
	 */
	public void addVertex(String vertId)
	{
		if(!this.containsVertId(vertId))
		{
			addVertex(new Vertex(vertId));
		}
	}
	
	/**
	 * 
	 * @param vertexId
	 */
	public Vertex removeVertex(String vertexId)
	{
		Collection<Edge> incidentEdges = edges.getIncidentEdges(vertexId);
		for(Edge incidentEdge : incidentEdges)
		{
			edges.remove(incidentEdge);
		}
		return vertices.remove(vertexId);
	}
	
	/**
	 * 
	 * @param vert
	 * @return
	 */
	public Vertex removeVertex(Vertex vert)
	{
		return removeVertex(vert.getId());
	}
	
	/**
	 * 
	 * @param edge
	 */
	//TODO: This is safe but ugly. Make more elegant.
	public boolean addEdge(Edge edge)
	{
		if(edge == null) return false;
		
		Vertex vert1 = edge.getVert1();
		Vertex vert2 = edge.getVert2();
		double weight = edge.getWeight();
		
		Vertex newVert1;
		if(!vertices.containsId(vert1.getId()))
		{
			vertices.add(vert1);
			newVert1 = vert1;
		}
		else if (!vertices.contains(vert1))
			newVert1 = vertices.getVertex(vert1.getId());
		else
			newVert1 = vert1;
		
		Vertex newVert2;
		if(!vertices.containsId(vert2.getId()))
		{
			vertices.add(vert2);
			newVert2 = vert2;
		}
		else if (!vertices.contains(edge.getVert2()))
			newVert2 = vertices.getVertex(vert2.getId());
		else 
			newVert2 = vert2;
		
		return edges.add(new Edge(newVert1, newVert2, weight));
	}
	
	/**
	 * 
	 * @param vert1
	 * @param vert2
	 * @param weight
	 * @return
	 */
	public boolean addEdge(Vertex vert1, Vertex vert2, double weight)
	{
		Edge edge = new Edge(vert1, vert2, weight);
		return addEdge(edge);
	}
	
	/**
	 * 
	 * @param vert1
	 * @param vert2
	 * @return
	 */
	public boolean addEdge(Vertex vert1, Vertex vert2)
	{
		Edge edge = new Edge(vert1, vert2);
		return addEdge(edge);
	}
	
	/**
	 * 
	 * @param vert1Id
	 * @param vert2Id
	 * @param weight
	 * @return
	 */
	public boolean addEdge(String vert1Id, String vert2Id, double weight)
	{
		Edge edge = new Edge(vert1Id, vert2Id, weight);
		return addEdge(edge);
	}
	
	/**
	 * 
	 * @param vert1Id
	 * @param vert2Id
	 * @return
	 */
	public boolean addEdge(String vert1Id, String vert2Id)
	{
		Edge edge = new Edge(vert1Id, vert2Id);
		return addEdge(edge);
	}
	
	/**
	 * 
	 * @param edgeId
	 * @return
	 */
	public boolean addEdge(String edgeId, double weight)
	{
		Edge edge = new Edge(edgeId, weight);
		return addEdge(edge);
	}
	
	/**
	 * 
	 * @param edgeId
	 * @return
	 */
	public boolean addEdge(String edgeId)
	{
		Edge edge = new Edge(edgeId);
		return addEdge(edge);
	}
	
	/**
	 * 
	 * @param edge
	 */
	public void removeEdge(Edge edge)
	{
		edges.remove(edge);
	}
	
	
	/**
	 * 
	 * @param edge
	 * @return
	 */
	public boolean contains(Edge edge)
	{
		return edges.contains(edge);
	}
	
	/**
	 * 
	 * @param vert
	 * @return
	 */
	public boolean contains(Vertex vert)
	{
		return vertices.contains(vert);
	}
	
	/**
	 * 
	 * @param edgeId
	 * @return
	 */
	public boolean containsEdgeId(String edgeId)
	{
		return edges.containsId(edgeId);
	}
	
	/**
	 * 
	 * @param vertId
	 * @return
	 */
	public boolean containsVertId(String vertId)
	{
		return vertices.containsId(vertId);
	}
	
	/**
	 * 
	 * @return
	 */
	public int getNumVerts()
	{
		return vertices.size();
	}
	
	/**
	 * 
	 * @return
	 */
	public int getNumEdges()
	{
		return edges.size();
	}
	
	/**
	 * 
	 * @param vertId
	 * @return
	 */
	public Collection<Vertex> getNeighbors(String vertId)
	{
		Set<Vertex> neighbors = new TreeSet<Vertex>();
		Collection<Edge> incidentEdges = edges.getIncidentEdges(vertId);
		for(Edge edge : incidentEdges)
		{
			neighbors.add(edge.getVert2());
		}
		return neighbors;
	}
	
	/**
	 * 
	 * @param vertId
	 * @return
	 */
	public Collection<Edge> getIncidentEdges(String vertId)
	{
		return edges.getIncidentEdges(vertId);
	}
	
	
	
	

}
