package graph;

import java.io.File;
import java.io.IOException;
import java.util.Comparator;
import java.util.zip.DataFormatException;


/**
 * Provides tools to work with a file containing graph information
 * in a specific format.
 *
 */
public abstract class GraphFile
{
	
	////////////
	// FIELDS //
	////////////
	
	protected final Graph graph;
	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * Constructs a GraphFile to work with the specified Graph.
	 * @param graph The Graph to represent in a specific file format
	 */
	public GraphFile(Graph graph)
	{
		this.graph = graph;
	}
	
	/**
	 * Constructs a new GraphFile with the same underlying graph as the
	 * specified GraphFile. This constructor will normally be used to
	 * convert between different file formats.
	 * @param gf The GraphFile to represent in a specific format.
	 */
	public GraphFile(GraphFile gf)
	{
		this.graph = gf.graph;
	}
	
	/**
	 * Constructs a GraphFile with the information read from the specified
	 * File.
	 * @param inputFile
	 * @throws IOException
	 * @throws DataFormatException 
	 */
	public GraphFile(File inputFile) throws IOException, DataFormatException
	{
		this.graph = new Graph();
		this.readFromFile(inputFile);
	}
	
	
	
	
	
	
	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////
	
	/**
	 * Gets the underlying graph being represented.
	 * @return the Graph represented by this GraphFile 
	 */
	public Graph getGraph()
	{
		return graph;
	}
	
		
	

	
	/////////////////////
	// PUBLIC METHODS //
	/////////////////////
	
	/**
	 * Reads the graph information in the specified File into
	 * the underlying Graph.
	 * @param inputFile
	 */
	//TODO: Make this private
	public abstract void readFromFile(File inputFile) throws IOException, DataFormatException;
	
	/**
	 * Writes the underlying graph information to the specified File
	 * in a specific file format.
	 * @param outputFile
	 */
	public abstract void writeToFile(File outputFile) throws IOException;
	
	
	
	
	
}
