package graph;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;




//TODO: Decide if getNewVerts, getNewEdges, etc. should be here
/**
 * 
 *
 */
public class GraphSequence implements Iterable<Graph>
{
	
	////////////
	// FIELDS //
	////////////
	
	private final Map<String,Graph> graphs;
	private final Set<String> vertIds;
	private final Set<String> edgeIds;
	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 */
	public GraphSequence()
	{
		graphs = new TreeMap<String,Graph>();
		vertIds = new TreeSet<String>();
		edgeIds = new TreeSet<String>();
	}
	
	
	
	

	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////
	
	/**
	 * 
	 * @return
	 */
	public Set<String> getSeqIds()
	{
		return graphs.keySet();
	}
	
	/**
	 * 
	 * @param seqId
	 * @return
	 */
	public Graph getGraph(String seqId)
	{
		return graphs.get(seqId);
	}
	
	/**
	 * 
	 * @return
	 */
	public Set<String> getVertIds()
	{
		return vertIds;
	}
	
	/**
	 * 
	 * @return
	 */
	public Set<String> getEdgeIds()
	{
		return edgeIds;
	}
	
	
	
	
	
	
	

	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * 
	 * @return
	 */
	public int size()
	{
		return graphs.size();
	}
	
	
	/**
	 * 
	 * @param graph
	 * @param seqId
	 */
	public void addGraph(Graph graph, String seqId)
	{
		graphs.put(seqId, graph);
		for(Vertex vert : graph.getVertices())
		{
			vertIds.add(vert.getId());
		}
		for(Edge edge : graph.getEdges())
		{
			edgeIds.add(edge.getId());
		}
	}

	
	/**
	 * 
	 * @param vert
	 * @param seqId
	 */
	public void addVertex(Vertex vert, String seqId)
	{
		if(!graphs.containsKey(seqId))
		{
			graphs.put(seqId, new Graph());
		}
		Graph graph = graphs.get(seqId);
		graph.addVertex(vert);
	}
	
	
	/**
	 * 
	 * @param edge
	 * @param seqId
	 */
	public void addEdge(Edge edge, String seqId)
	{
		if(!graphs.containsKey(seqId))
		{
			graphs.put(seqId, new Graph());
		}
		Graph graph = graphs.get(seqId);
		graph.addEdge(edge);
	}
	
	
	//TODO: Check if should be kept as is, if so modify getEdgeFreqs to use it
	public Map<String,int[]> getEdgeOccurrences()
	{
		Map<String,int[]> occur = new TreeMap<String,int[]>();
		for(String edgeId : edgeIds)
		{
			int seqNum = 1;
			int count = 0;
			int lastOccur = 0;
			for(Graph graph : graphs.values())
			{
				if(graph.containsEdgeId(edgeId))
				{
					count++;
					lastOccur = seqNum;
				}
				seqNum++;
			}
			int[] vals = new int[2];
			vals[0] = count;
			vals[1] = lastOccur;
			occur.put(edgeId, vals);
		}
		return occur;
	}
	
	
	/**
	 * Calculates the frequency for every edge in the GraphSequence.
	 * @return a map with key as edgeId and value as edge frequency
	 */
	public Map<String,Double> getEdgeFreqs()
	{
		Map<String,Double> freqs = new TreeMap<String,Double>();
		for(String edgeId : edgeIds)
		{
			double occur = 0;
			for(Graph graph : graphs.values())
			{
				if(graph.containsEdgeId(edgeId))
					occur++;
			}
			freqs.put(edgeId, occur/graphs.size());
		}
		return freqs;
	}
	
	
	/**
	 * 
	 * @param graph1
	 * @param graph2
	 * @return
	 */
	public static Vertices getNewVerts(Graph graph1, Graph graph2)
	{
		Vertices newVerts = new Vertices();
		
		Vertices currVerts = graph1.getVertices();
		Vertices nextVerts = graph2.getVertices();
		for(Vertex nextVert : nextVerts)
		{
			if(!currVerts.contains(nextVert))
				newVerts.add(nextVert);
		}
		return newVerts;
	}
	
	
	/**
	 * 
	 * @param graph1
	 * @param graph2
	 * @return
	 */
	public static Vertices getOldVerts(Graph graph1, Graph graph2)
	{
		Vertices oldVerts = new Vertices();
		Vertices currVerts = graph1.getVertices();
		Vertices nextVerts = graph2.getVertices();
		for(Vertex currVert : currVerts)
		{
			if(!nextVerts.contains(currVert))
				oldVerts.add(currVert);
		}
		return oldVerts;
	}
	
	/**
	 * 
	 * @param graph1
	 * @param graph2
	 * @return
	 */
	public static Edges getNewEdges(Graph graph1, Graph graph2)
	{
		Edges newEdges = new Edges();
		Edges currEdges = graph1.getEdges();
		Edges nextEdges = graph2.getEdges();
		Iterator<Edge> edgeItr = nextEdges.iterator();
		while(edgeItr.hasNext())
		{
			Edge nextEdge = edgeItr.next();
			if(!currEdges.contains(nextEdge))
				newEdges.add(nextEdge);
		}
		return newEdges;
	}
	
	/**
	 * 
	 * @param graph1
	 * @param graph2
	 * @return
	 */
	public static Edges getOldEdges(Graph graph1, Graph graph2)
	{
		Edges oldEdges = new Edges();
		Edges currEdges = graph1.getEdges();
		Edges nextEdges = graph2.getEdges();
		Iterator<Edge> edgeItr = currEdges.iterator();
		while(edgeItr.hasNext())
		{
			Edge currEdge = edgeItr.next();
			if(!nextEdges.contains(currEdge))
				oldEdges.add(currEdge);
		}
		return oldEdges;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Iterable#iterator()
	 */
	@Override
	public Iterator<Graph> iterator()
	{
		return graphs.values().iterator();
	}
	

	
	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	

}
