package graphstream;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.zip.DataFormatException;
import graph.Graph;
import graph.GraphFile;
import graph.Vertex;

public class DGS extends GraphFile
{
	
	///////////////
	// CONSTANTS //
	///////////////
		
	private static final String FILE_FORMAT 	  = "DGS002";
	private static final int	DEFAULT_NUM_STEPS  = 0;
	private static final int 	DEFAULT_NUM_EVENTS = 0;
	

	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 * @param gf
	 */
	public DGS(Graph g)
	{
		super(g);
	}
	
	/**
	 * 
	 * @param gf
	 */
	public DGS(GraphFile gf)
	{
		super(gf);
	}
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException
	 * @throws DataFormatException
	 */
	public DGS(File inputFile) throws IOException, DataFormatException
	{
		super(inputFile);
		
	}

	
	
	
	

	@Override
	public void readFromFile(File inputFile) throws IOException, DataFormatException
	{
		// TODO Auto-generated method stub
		
	}

	
	/**
	 * 
	 */
	@Override
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = null;
		try
		{
			writer = new BufferedWriter(new FileWriter(outputFile));
			writeToFile(writer);
		}
		finally
		{
			if(writer != null)
				writer.close();
		}	
	}
	/**
	 * 
	 * @param writer
	 * @throws IOException 
	 */
	private void writeToFile(BufferedWriter writer) throws IOException
	{
		writeHeader(writer);
		writeNodes(writer);
	}


	/**
	 * 
	 * @param writer
	 * @throws IOException
	 */
	private void writeHeader(BufferedWriter writer) throws IOException
	{
		writer.write(FILE_FORMAT);
		writer.newLine();
		
		StringBuffer sb = new StringBuffer();
		sb.append(graph.getId() + " ");
		sb.append(DEFAULT_NUM_STEPS + " ");
		sb.append(DEFAULT_NUM_EVENTS);
		writer.write(sb.toString());
		writer.newLine();
		writer.newLine();
	}
	
	/**
	 * 
	 * @param writer
	 * @throws IOException 
	 */
	private void writeNodes(BufferedWriter writer) throws IOException
	{
		for(Vertex v : graph.getVertices())
		{
			String Id = v.getId();
			String label = v.getAttrVal("label");
			double r = Double.parseDouble(v.getAttrVal("r")) * 255;
			double g = Double.parseDouble(v.getAttrVal("g")) * 255;
			double b = Double.parseDouble(v.getAttrVal("b")) * 255;
			
			StringBuffer sb = new StringBuffer();
			sb.append("an " + Id);
			sb.append(" label=" + label);
			sb.append(" ui.stylesheet=");
			sb.append("fill-color: rgb(");
			sb.append(r + ", ");
			sb.append(g + ", ");
			sb.append(b + ");");
			writer.write(sb.toString());
			writer.newLine();
		}
		
	}

	

	////////////
	// FIELDS //
	////////////

	

	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////

	////////////////////
	// PUBLIC METHODS //
	////////////////////

	/////////////////////
	// PRIVATE METHODS //
	/////////////////////

	//////////////////////
	// STANDARD METHODS //
	//////////////////////

}
