package graphview;


import graphview.Wadj;
import graph.Edge;
import graph.Graph;
import graph.GraphFile;
import graph.Vertex;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.TreeMap;




/**
 * 
 * @author nparikh
 *
 */
public class Twadj 
{
	
	///////////////
	// CONSTANTS //
	///////////////
	
	private static final String LINE_DELIMITER = getLineDelimiter(); 
	

	
	
	
	////////////
	// FIELDS //
	////////////
	
	private final Map<Integer,List<Edge>> twadjMap;
	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 */
	public Twadj()
	{
		this.twadjMap = new TreeMap<Integer,List<Edge>>();
	}
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException
	 */
	public Twadj(File inputFile) throws IOException
	{
		this();
		this.readFromFile(inputFile);
	}

	/**
	 * 
	 * @param timestampedWadjs
	 */
	//TODO: Make work with new version of Graph and Edges
	public Twadj(Map<Integer,GraphFile> graphFiles)
	{
		this();
//		for(Integer timestamp : graphFiles.keySet())
//		{
//			Graph graph = graphFiles.get(timestamp).getGraph();
//			for(Edge edge : graph.getEdges())
//			{
//				addEdge(timestamp, edge);
//			}
//		}
	}
	
	
	
	
	
	
	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////
	
	/**
	 * 
	 * @return
	 */
	public Map<Integer,List<Edge>> getTwadjMap()
	{
		return twadjMap;
	}
	
	/**
	 * 
	 * @return
	 */
	//TODO: Change method to work with graphs
	public Map<Integer,Wadj> getTimestampedWadjs()
	{
		Map<Integer,Wadj> timestampedWadjs = new TreeMap<Integer,Wadj>();
		for(Integer timestamp : twadjMap.keySet())
		{
			//timestampedWadjs.put(timestamp, new Wadj(twadjMap.get(timestamp)));
		}
		return timestampedWadjs;
	}
	
	
	
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * 
	 * @param timestamp
	 */
	public void addTimestamp(Integer timestamp)
	{
		twadjMap.put(timestamp, new ArrayList<Edge>());
	}
	
	/**
	 * 
	 * @param timestamp
	 * @param edge
	 */
	public void addEdge(Integer timestamp, Edge edge)
	{
		if(!twadjMap.containsKey(timestamp))
			addTimestamp(timestamp);
		List<Edge> edges = twadjMap.get(timestamp);
		edges.add(edge);
	}
	
	/**
	 * 
	 * @param file
	 * @throws IOException 
	 */
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));
		for(Integer timestamp : twadjMap.keySet())
		{
			writer.write(timestamp + ":");
			List<Edge> edges = twadjMap.get(timestamp);
			for(Edge edge : edges)
			{
				writer.write(edge.getVert1Id() + "," + edge.getVert2Id() + " ");
			}
			writer.newLine();
			writer.flush();
		}
	}
	
	
	
	
	
	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException 
	 */
	private void readFromFile(File inputFile) throws IOException 
	{
		BufferedReader reader = new BufferedReader(new FileReader(inputFile));
		String line;
		while((line = reader.readLine()) != null)
		{
			Scanner scanner = new Scanner(line.trim());
			scanner.useDelimiter(LINE_DELIMITER);
			Integer timestamp = scanner.nextInt();
			while(scanner.hasNext())
			{
				Vertex vert1 = new Vertex(scanner.next());
				Vertex vert2 = new Vertex(scanner.next());
				Edge edge = new Edge(vert1, vert2);
				addEdge(timestamp, edge);
			}
		}
	}
	
	/**
	 * 
	 * @return
	 */
	private static String getLineDelimiter()
	{
		//TODO: need to make "greedy" about whitespace characters as delimiter.
		StringBuffer sb = new StringBuffer();
		sb.append("\\s+");
		sb.append("|\\s*:\\s*");
		sb.append("|\\s*\\,\\s*");
		return sb.toString();
	}
	
	
	
	
	
	
	
	
	
}
