package graphview;

import graph.Atf;
import graph.Attributes;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;
import java.util.TreeMap;





/**
 * 
 * @author nparikh
 *
 */
public class WadjLabels
{
	
	///////////////
	// CONSTANTS //
	///////////////
	
	private static final String FILE_EXT = ".wadj.labels";
	private static final String DELIMITERS = "\\[|\\]|=|;"; 
	private static final String TAG_VAL_REGEX = "\"[^\"]+\"";
	private static final String DEFUALT_LABEL = "default_label";

	
	
	
	
	////////////
	// FIELDS //
	////////////

	private final Map<String, String> labels;
	private final Atf tags;





	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////

	/**
	 * 
	 */
	public WadjLabels()
	{
		labels = new TreeMap<String, String>();
		tags = new Atf("VERTEX");
	}
	
	/**
	 * 
	 * @param tags
	 */
	public WadjLabels(Atf tags)
	{
		labels = new TreeMap<String, String>();
		this.tags = tags;
	}

	/**
	 * 
	 * @param inputFile
	 * @throws IOException
	 */
	public WadjLabels(File inputFile) throws IOException
	{
		this();
		readFromFile(inputFile);
	}
	

	
	
	

	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////

	/**
	 * 
	 * @param vertId
	 * @return
	 */
	public String getLabel(String vertId)
	{
		return labels.get(vertId);
	}
	
	/**
	 * 
	 * @param vertId
	 * @param tagId
	 * @return
	 */
	public String getTagValue(String vertId, String tagId)
	{
		return tags.getValue(vertId, tagId);
	}
	
	/**
	 * 
	 * @return
	 */
	public Collection<String> getVertIds()
	{
		return labels.keySet();
	}
	
	/**
	 * 
	 * @param vertId
	 * @return
	 */
	public Collection<String> getTagIds(String vertId)
	{
		return tags.getAttrIds(vertId);
	}
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////

	/**
	 * 
	 * @param vertId
	 * @param label
	 */
	public void addLabel(String vertId, String label)
	{
		labels.put(vertId, label);
	}
	
	
	/**
	 * 
	 * @param vertId
	 * @param tagId
	 * @param tagVal
	 */
	public void addTag(String vertId, String tagId, String tagVal)
	{
		if(!labels.containsKey(vertId))
		{
			labels.put(vertId, DEFUALT_LABEL);
		}
		tags.add(vertId, tagId, tagVal);
	}
	
	
	/**
	 * Writes the stored data to the specified outputFile.
	 * @param outputFile
	 * @throws IOException 
	 */
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));
		StringBuffer sb;
		for(String vertexId : labels.keySet())
		{
			sb = new StringBuffer();
			sb.append(vertexId + " [label=\"");
			sb.append(labels.get(vertexId) + "\"");
			
			Attributes attrs = tags.getAttrs(vertexId);
			if(attrs != null)
			{
				for(String tagId : attrs.getIds())
				{
					String tagVal = attrs.getValue(tagId);
					sb.append(" " + tagId + "=\"");
					sb.append(tagVal + "\"");
				}
			}
			
			sb.append("];");
			writer.write(sb.toString() + "\n");
			writer.flush();
		}
		writer.close();
	}


	/**
	 * 
	 * @param inputFile
	 * @throws IOException 
	 */
	public void readFromFile(File inputFile) throws IOException
	{
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(new FileReader(inputFile));
			String line = null;
			while((line = reader.readLine()) != null) 
			{
				parseLine(line);
			}
		}
		finally
		{
			if(reader != null)
				reader.close();
		}
	}

	





	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * 
	 * @param line
	 */
	private void parseLine(String line)
	{
		line = line.replaceAll(DELIMITERS, " ");

		Scanner sc = new Scanner(line);
		String vertId = sc.next();
		String label = sc.findInLine(TAG_VAL_REGEX);
		label = label.replaceAll("\"", "");
		labels.put(vertId, label);
		
		while(sc.hasNext())
		{
			String tagId = sc.next();
			String tagVal = sc.findInLine(TAG_VAL_REGEX);
			tagVal = tagVal.replaceAll("\"", "");
			tags.add(vertId, tagId, tagVal);
		}
		
	}



}
