package utils;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;


//TODO: Add support for alpha channel to create RGBA colors
public class ColorMapper
{
	
	///////////////
	// CONSTANTS //
	///////////////
	
	private static final int MAX_VALUE = 255;
	
	
	
	////////////
	// FIELDS //
	////////////
	
	List<Color> colors;
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * Initializes the color map with RGB color values 
	 * in order from blue to red.
	 * 
	 */
	public ColorMapper()
	{
		this.colors = new ArrayList<Color>();
		this.init();
	}
	
	
	
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * Takes a range (min, max) and maps the specified value 
	 * to a corresponding RGB value. 
	 * @param min
	 * @param max
	 * @param value must be >= to min and <= max
	 * @return
	 */
	//TODO: should throw an exception instead of printing error
	public Color getColor(double min, double max, double value)
	{
		if(value < min || value > max)
		{
			System.err.printf("ColorMapper Error: The value %f ", value);
			System.err.printf("is out of the range [%f, %f]%n", value, min, max);
			return null;
		}
		
		double shiftedMax = max - min;
		double shiftedValue = value - min;
		
		int maxIndex = colors.size()-1;
		double scale = maxIndex/shiftedMax;
		int colorIndex = (int)Math.floor(shiftedValue * scale);
		return (colors.get(colorIndex));
	}
	
	
	
	
	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * 
	 */
	private void init()
	{
		// blue -> aqua
		for (int i = 0; i <= MAX_VALUE; i++)
		{	
			this.colors.add(new Color(0, i, MAX_VALUE));
		}
		// aqua -> green
		for (int i = MAX_VALUE-1; i >=0; i--)
		{	
			this.colors.add(new Color(0, MAX_VALUE, i));
		}
		// green -> yellow
		for (int i = 1; i <= MAX_VALUE; i++)
		{	
			this.colors.add(new Color(i, MAX_VALUE, 0));
		}
		// yellow -> red
		for (int i = MAX_VALUE-1; i >= 0; i--)
		{	
			this.colors.add(new Color(MAX_VALUE, i, 0));
		}
	}
	
	

}
