package utils;

import static graph.GraphElemType.*;
import graph.Atf;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;
import java.util.Scanner;
import java.util.TreeMap;





public class XY
{
	
	
	///////////////
	// CONSTANTS //                                                                                                                                                                                                                                                                         
	///////////////
	
	private static final String FILE_EXTENSION = ".xy";
	

	
	
	
	////////////
	// FIELDS //
	////////////
	
	private Map<String,Point2D> pointsMap; 
	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////

	/**
	 * 
	 * @param inputFile
	 * @throws IOException
	 */
	public XY()
	{
		pointsMap = new TreeMap<String,Point2D>();
	}
	
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException
	 */
	public XY(File inputFile) throws IOException
	{
		super();
		this.readFromFile(inputFile);
	}
	
	/**
	 * 
	 * @param atf
	 */
	public XY(Atf atf)
	{
		super();
		if(atf.getType() != VERTEX)
		{
			//TODO: This should throw an exception instead.
			System.err.println("Atf must be of type VERTEX");
		}
		this.readFromAtf(atf);
	}

	
	
	
	
	
	
	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////
	
	/**
	 * 
	 * @return
	 */
	public String getFileExtension()
	{
		return FILE_EXTENSION;
	}
	
	


	

	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * 
	 * @param vertexId
	 * @param x
	 * @param y
	 */
	public void add(String id, double x, double y)
	{
		Point2D coords = new Point2D.Double(x, y);
		pointsMap.put(id, coords);
	}
	
	
	
	/**
	 * 
	 * @param factor
	 * @return
	 */
	public XY scale(double factor)
	{
		XY scaled = new XY();
		for(String id : pointsMap.keySet())
		{
			Point2D coords = pointsMap.get(id);
			double scaledX = coords.getX() * factor;
			double scaledY = coords.getY() * factor;
			this.add(id, scaledX, scaledY);
		}
		return scaled;
	}
	
	/**
	 * 
	 * @param dx
	 * @param dy
	 * @return
	 */
	public XY translate(double dx, double dy)
	{
		XY translated = new XY();
		for(String id : pointsMap.keySet())
		{
			Point2D coords = pointsMap.get(id);
			double transX = coords.getX() + dx;
			double transY = coords.getY() + dy;
			translated.add(id, transX, transY);
		}
		return translated;
	}
	
	
	
	
	
	/**
	 * 
	 * @param outputFile
	 * @throws IOException
	 */
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));
		for(String id : pointsMap.keySet())
		{
			Point2D coords = pointsMap.get(id);
			writer.write(id + " " + coords.getX() + " " + coords.getY());
			writer.newLine();
			writer.flush();
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public Atf toAtf()
	{
		Atf atf = new Atf(VERTEX);
		
		for(String id : pointsMap.keySet())
		{
			Point2D point = pointsMap.get(id);
			atf.add(id, "x", Double.toString(point.getX()));
			atf.add(id, "y", Double.toString(point.getY()));
		}
		
		return atf;
	}
	
	
	
	
	
	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException 
	 */
	public void readFromFile(File inputFile) throws IOException 
	{
		BufferedReader reader = new BufferedReader(new FileReader(inputFile));
		String line;
		while((line = reader.readLine()) != null)
		{
			Scanner scanner = new Scanner(line);
			String id = scanner.next();
			double x = scanner.nextDouble();
			double y = scanner.nextDouble();
			this.add(id, x, y);
		}
	}
	
	
	/**
	 * 
	 * @param atf
	 */
	public void readFromAtf(Atf atf)
	{
		for(String id : atf.getElemIds())
		{
			String xStr = atf.getValue(id, "x");
			String yStr = atf.getValue(id, "y");
			if(xStr != null && yStr != null)
			{
				double x = Double.parseDouble(atf.getValue(id, "x"));
				double y = Double.parseDouble(atf.getValue(id, "y"));
				this.add(id, x, y);
			}
		}
	}
	
	
}
